#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>
#include <grp.h>

struct i_flags {
    int cg;
    int g;
    int u;
};

void id(struct passwd *pwd)
{
    int i, ngroups = 0;
    gid_t *groups = NULL;
    struct group *grp;
    
    getgrouplist(pwd->pw_name, pwd->pw_gid, groups, &ngroups);
    groups = calloc(sizeof (gid_t), ngroups);
    getgrouplist(pwd->pw_name, pwd->pw_gid, groups, &ngroups);
    grp = getgrgid(pwd->pw_gid);
    
    printf("uid=%d(%s) gid=%d(%s) groups=", pwd->pw_uid, pwd->pw_name, 
	   pwd->pw_gid, grp->gr_name);
    for (i = 0; i < ngroups; i++) {
	if (i > 0)
	    printf(",");
	grp = getgrgid(groups[i]);
	if (grp != NULL)
	    printf("%d(%s)", groups[i], grp->gr_name);
    }
    printf("\n");
}

void printgroups(struct passwd *pwd)
{
    int i, ngroups = 0;
    gid_t *groups = NULL;

    getgrouplist(pwd->pw_name, pwd->pw_gid, groups, &ngroups);
    groups = calloc(sizeof (gid_t), ngroups);
    getgrouplist(pwd->pw_name, pwd->pw_gid, groups, &ngroups);

    for (i = 0; i < ngroups; i++) {
	if (i > 0)
	    printf(" ");
	printf("%d", (int)groups[i]);
    }
    
    printf("\n");
}

int main(int argc, char **argv)
{
    int opt;
    struct i_flags *flags;
    struct passwd *pwd;

    flags = calloc(sizeof (struct i_flags), 1);
    
    while ((opt = getopt(argc, argv, "Ggnur")) != -1) {
	switch (opt) {
	case 'G': flags->cg = 1; break;
	case 'g': flags->g = 1; break;
	case 'u': flags->u = 1; break;
	case '?': return 1;
	}
    }
    if (flags->g + flags->cg + flags->u > 1) {
	fprintf(stderr, "%s: cannot print only of more than one choice",
		argv[0]);
	return 1;
    }
    if (optind < argc - 1) {
	fprintf(stderr, "%s: extra operands\n", argv[0]);
	return 1;
    } else if (optind == argc - 1) {
	if ((pwd = getpwnam(argv[optind])) == NULL) {
	    fprintf(stderr, "%s: %s: No such user\n", argv[0], argv[optind]);
	    return 1;
	}
    } else
	pwd = getpwuid(getuid());

    if (flags->u)
	printf("%d\n", pwd->pw_uid);
    else if (flags->g)
	printf("%d\n", pwd->pw_gid);
    else if (flags->cg)
	printgroups(pwd);
    else
	id(pwd);

    return 0;
}
