#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <getopt.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <pwd.h>
#include <grp.h>
#include <time.h>

struct opts
{
    char a;
    char l;
};

struct opts options;

void parse_options(int argc, char** argv)
{
    int c;
    options.a = 0;
    options.l = 0;
    for(;;)
    {
        c = getopt(argc, argv, "la");
        if (c == -1)
            break;

        switch (c)
        {
            case 'a':
                options.a = 1;
                break;

            case 'l':
                options.l = 1;
                break;

            case '?':
                exit (1);
        }
    }
    return;
}

int selector(const struct dirent* dirent)
{
    if ((options.a) || (dirent->d_name[0] != '.'))
        return 1;
    else
        return 0;
}

char* get_permissions(mode_t mode)
{
    int i;
    char* perms = malloc(10 * sizeof(char));

    for (i = 0; i < 9; i++)
        perms[i] = '-';

    if (mode & S_IRUSR)
        perms[0] = 'r';
    
    if (mode & S_IWUSR)
        perms[1] = 'w';
    
    if (mode & S_ISUID)
        if (mode & S_IXUSR)
            perms[2] = 'S';
        else
            perms[2] = 's';
    else
        if (mode & S_IXUSR)
            perms[2] = 'x';
    
    
    if (mode & S_IRGRP)
        perms[3] = 'r';
    
    if (mode & S_IWGRP)
        perms[4] = 'w';
    
    if (mode & S_ISGID)
        if (mode & S_IXGRP)
            perms[5] = 'S';
        else
            perms[5] = 's';
    else
        if (mode & S_IXGRP)
            perms[5] = 'x';
    
    
    if (mode & S_IROTH)
        perms[6] = 'r';
    
    if (mode & S_IWOTH)
        perms[7] = 'w';
    
    if (mode & S_ISVTX)
        if (mode & S_IXOTH)
            perms[8] = 'T';
        else
            perms[8] = 't';
    else
        if (mode & S_IXOTH)
            perms[8] = 'x';

    perms[9] = '\0';

    return perms;
}

char get_filetype(mode_t mode)
{
    if (S_ISDIR(mode))
        return 'd';

    if (S_ISCHR(mode))
        return 'c';

    if (S_ISBLK(mode))
        return 'b';

    if (S_ISREG(mode))
        return '-';

    if (S_ISFIFO(mode))
        return 'p';

    if (S_ISLNK(mode))
        return 'l';

    if (S_ISSOCK(mode))
        return 's';

    return '0';
}

void print_entries(int n, struct dirent** entries)
{
    int i;
    int count = 0;
    struct stat* file = malloc(sizeof(struct stat));
    char file_type;
    char* file_perms = NULL;
    struct passwd* pwuid = NULL;
    struct group* grp = NULL;
    struct tm* time = NULL;

    if (!options.l)
    {
        for(i = 0; i < n; i++)
        {
            if (count > 60)
            {
                printf("\n");
                count = 0;
            }
            printf("%s  ", entries[i]->d_name);
            count += strlen(entries[i]->d_name) + 2;
        }
        printf("\n");
    }
    else
    {
        for(i = 0; i < n; i++)
        {
            stat(entries[i]->d_name, file);
            count += file->st_blocks;
        }
        printf("total %i\n", (count / 2));
        for(i = 0; i < n; i++)
        {
            stat(entries[i]->d_name, file);
            file_type = get_filetype(file->st_mode);
            file_perms = get_permissions(file->st_mode);
            pwuid = getpwuid(file->st_uid);
            grp = getgrgid(file->st_gid);
            time = localtime(&file->st_mtime);
            
            printf("%c%s ", file_type, file_perms);
            printf("%d ", (int)file->st_nlink);
            if (pwuid != NULL)
                printf("%s ", pwuid->pw_name);
            else
                printf("%d ", file->st_uid);
            if (grp != NULL)
                printf("%s ", grp->gr_name);
            else
                printf("%d ", file->st_gid);
            printf("%d ", (int)file->st_size);
            printf("%d-%d-", time->tm_year + 1900, time->tm_mon + 1);
            printf("%d %d:", time->tm_mday, time->tm_hour);
            printf("%d %s\n", time->tm_min, entries[i]->d_name);
        }
    }
}


int main(int argc, char** argv)
{
    struct dirent** entries = NULL;
    int n;

    parse_options(argc, argv);

    if (optind < argc)
        chdir(argv[optind]);
    n = scandir("./", &entries, selector, alphasort);
    if (n >= 0)
    {
        print_entries(n, entries);
        return 0;
    }
    else
    {
        puts("Unable to scan directory");
        return 1;
    }
}
