#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
#include "wc.h"

int main(int argc, char **argv)
{
    struct s_wc *wc = NULL, *total = NULL;
    struct wc_flags *flags = NULL;
    int i;
    
    flags = calloc(sizeof (struct wc_flags), 1);
    flags->l = flags->w = flags->c = 1;
    wc = calloc(sizeof (struct s_wc), 1);
    total = calloc(sizeof (struct  s_wc), 1);

    if (option_parser(argc, argv, flags) == 1) {
      usage(EXIT_FAILURE);
      return EXIT_FAILURE;
    }
    if (flags->h) {
	usage(EXIT_SUCCESS);
	return EXIT_SUCCESS;
    }
    if (flags->v) {
	puts("wc (Lemur) 1664.42.69");
	return EXIT_SUCCESS;
    }
    if (argc == optind) {
        fwc("-", wc);
	check_flags(flags, wc);
	puts("");
    } else {
	for (i = optind; i < argc; i++) {
	    if (fwc(argv[i], wc)) {
		perror(argv[0]);
		usage(EXIT_FAILURE);
		return EXIT_FAILURE;
	    }
	    check_flags(flags, wc);
	    printf(" %s\n", argv[i]);
	    total->l += wc->l;
	    total->w += wc->w;
	    total->c += wc->c;
	}
	if (argc > optind + 1) {
	    check_flags(flags, total);
	    printf(" total\n");
	}
    }

    return EXIT_SUCCESS;
}

void usage(int status)
{
    if (status != EXIT_SUCCESS)
	fprintf(stderr, "Try `wc --help' for more information.\n");
    else {
	puts("Usage: wc [OPTION].. [FILE]...\n\
Print newline, word, and byte counts for each FILE, and a total line if\n\
more than one FILE is specified.  With no FILE, or when FILE is -,\n\
read standard input.\n\
  -c            print the byte counts\n\
  -m            print the character counts\n\
  -l            print the newline counts\n\
  -w            print the word counts\n\
  -h            display this help and exit\n\
  -v            output version information and exit");
    }
}

void init_wc(struct s_wc *wc)
{
    wc->l = 0;
    wc->w = 0;
    wc->c = 0;
}

int fwc(const char filename[], struct s_wc *wc)
{
    FILE *fp;
    char c;
    int words = 0;
    
    init_wc(wc);
    if (!strcmp(filename, "-")) {
	while ((c = fgetc(stdin)) != EOF) {
	    if (c == '\n')
		wc->l++;
	    if (isspace(c)) {
		if (words == 0)
		    wc->w++;
		words = 1;
	    } else {
		words = 0;
	    }
	    wc->c++;
	}
    } else {
	fp = fopen(filename, "r");
	if (fp == NULL)
	    return 1;
	while ((c = fgetc(fp)) != EOF) {
	    if (c == '\n')
		wc->l++;
	    if (isspace(c)) {
		if (words == 0)
		    wc->w++;
		words = 1;
	    } else {
		words = 0;
	    }
	    wc->c++;
	}
	fclose(fp);
    }
    return 0;
}

int option_parser(int argc, char **argv, struct wc_flags *flags)
{
    int opt;
    
    int i = 0;
    
    while ((opt = getopt(argc, argv, "lwcmhv")) != EOF) {
	i++;
	if (i == 1)
	    flags->l = flags->w = flags->m = flags->c = 0;
	switch (opt) {
	case 'l': flags->l = 1; break;
	case 'w': flags->w = 1; break;
	case 'c': flags->c = 1; break;
	case 'm': flags->m = 1; break;
	case 'h': flags->h = 1; break;
	case 'v': flags->v = 1; break;
	case '?': return 1;
	}
    }

    return 0;
}

void check_flags(struct wc_flags *flags, struct s_wc *wc)
{
    if (flags->l + flags->w + flags->m + flags->c == 1) {
	if (flags->l)
	    printf("%d", wc->l);
	if (flags->w)
	    printf("%d", wc->w);
	if (flags->m)
	    printf("%d", wc->c);
	if (flags->c)
	    printf("%d", wc->c);
    } else {
	if (flags->l)
	    printf("%7d", wc->l);
	if (flags->w)
	    printf("%7d", wc->w);
	if (flags->m)
	    printf("%7d", wc->c);
	if (flags->c)
	    printf("%7d", wc->c);
    }
}
